#include "intersect.h"

#include <stdio.h>
#include <math.h>

int circle_circle_intersection(double x0, double y0, double r0,
                               double x1, double y1, double r1,
                               double *xi, double *yi,
                               double *xi_prime, double *yi_prime)
{
  double a, dx, dy, d, h, rx, ry;
  double x2, y2;

  /* dx and dy are the vertical and horizontal distances between
   * the circle centers.
   */
  dx = x1 - x0;
  dy = y1 - y0;

  /* Determine the straight-line distance between the centers. */
  //d = sqrt((dy*dy) + (dx*dx));
  d = hypot(dx,dy); // Suggested by Keith Briggs

  /* Check for solvability. */
  if (d > (r0 + r1))
  {
    /* no solution. circles do not intersect. */
    return 0;
  }
  if (d < fabs(r0 - r1))
  {
    /* no solution. one circle is contained in the other */
    return 0;
  }

  /* 'point 2' is the point where the line through the circle
   * intersection points crosses the line between the circle
   * centers.
   */

  /* Determine the distance from point 0 to point 2. */
  a = ((r0*r0) - (r1*r1) + (d*d)) / (2.0 * d) ;

  /* Determine the coordinates of point 2. */
  x2 = x0 + (dx * a/d);
  y2 = y0 + (dy * a/d);

  /* Determine the distance from point 2 to either of the
   * intersection points.
   */
  h = sqrt((r0*r0) - (a*a));

  /* Now determine the offsets of the intersection points from
   * point 2.
   */
  rx = -dy * (h/d);
  ry = dx * (h/d);

  /* Determine the absolute intersection points. */
  *xi = x2 + rx;
  *xi_prime = x2 - rx;
  *yi = y2 + ry;
  *yi_prime = y2 - ry;

  return 1;
}





/*
// http://www.idevgames.com/forum/archive/index.php/t-7458.html
// Joseph Duchesne
int line_line_intersection(float ax,float ay, float bx, float by, float cx, float cy, float dx, float dy)
{
	float tarray[4][2]; //<===== Find the inner bounding of rect ABCD

	if (ax<bx){
		tarray[0][0]=ax;
		tarray[1][0]=bx;
	}else{
		tarray[0][0]=bx;
		tarray[1][0]=ax;
	}

	if (ay<by){
		tarray[0][1]=ay;
		tarray[1][1]=by;
	}else{
		tarray[0][1]=by;
		tarray[1][1]=ay;
	}

	if (cx<dx){
		tarray[2][0]=cx;
		tarray[3][0]=dx;
	}else{
		tarray[2][0]=dx;
		tarray[3][0]=cx;
	}
	if (cy<dy){
		tarray[2][1]=cy;
		tarray[3][1]=dy;
	}else{
		tarray[2][1]=dy;
		tarray[3][1]=cy;
	}

	float tarray2[2][2]; //<===== Sort Section Two
	if (tarray[0][0]<tarray[2][0]) { tarray2[0][0]=tarray[2][0]; }else{ tarray2[0][0]=tarray[0][0]; } //biggest of the small ones
	if (tarray[0][1]<tarray[2][1]) { tarray2[0][1]=tarray[2][1]; }else{ tarray2[0][1]=tarray[0][1]; } //biggest of the small ones
	if (tarray[1][0]<tarray[3][0]) { tarray2[1][0]=tarray[1][0]; }else{ tarray2[1][0]=tarray[3][0]; } //biggest of the small ones
	if (tarray[1][1]<tarray[3][1]) { tarray2[1][1]=tarray[1][1]; }else{ tarray2[1][1]=tarray[3][1]; } //biggest of the small ones

	float mab=(ay-by)/(ax-bx); //<===== Find Slopes of Lines
	float mcd=(cy-dy)/(cx-dx);
	
	if (mab==mcd) return 0; //the lines are paralell
	float yiab=((ax-bx)*ay-ax*(ay-by))/(ax-bx); //<===== Find the y Intercept of Lines
	float yicd=((cx-dx)*cy-cx*(cy-dy))/(cx-dx);
	float x=(yicd-yiab)/(mab-mcd); //<===== Find the Intersection Points of the Line
	float y=mab*x+yiab;
	if (x>tarray2[0][0] && x<tarray2[1][0] && y>tarray2[0][1] && y<tarray2[1][1]) { return 1; }else{ return 0; }
}
*/

//http://tog.acm.org/GraphicsGems/gemsiii/insectc.c
/* Faster Line Segment Intersection   */
/* Franklin Antonio                   */

/* return values */
#define DONT_INTERSECT 0
#define DO_INTERSECT   1
#define PARALLEL       2


/* The SAME_SIGNS macro assumes arithmetic where the exclusive-or    */
/* operation will work on sign bits.  This works for twos-complement,*/
/* and most other machine arithmetic.                                */
#define SAME_SIGNS( a, b ) \
	(((long) ((unsigned long) a ^ (unsigned long) b)) >= 0 )


/* The use of some short working variables allows this code to run   */
/* faster on 16-bit computers, but is not essential.  It should not  */
/* affect operation on 32-bit computers.  The short working variables*/
/* to not restrict the range of valid input values, as these were    */
/* constrained in any case, due to algorithm restrictions.           */


int line_line_intersection(long x1, long y1, long x2, long y2, long x3, long y3, long x4, long y4) 
{
	long t1, t2;
	long *x = &t1, *y = &t2;

long Ax,Bx,Cx,Ay,By,Cy,d,e,f,num,offset;
short x1lo,x1hi,y1lo,y1hi;

Ax = x2-x1;
Bx = x3-x4;

if(Ax<0) {						/* X bound box test*/
  x1lo=(short)x2; x1hi=(short)x1;
  } else {
  x1hi=(short)x2; x1lo=(short)x1;
  }
if(Bx>0) {
  if(x1hi < (short)x4 || (short)x3 < x1lo) return DONT_INTERSECT;
  } else {
  if(x1hi < (short)x3 || (short)x4 < x1lo) return DONT_INTERSECT;
  }

Ay = y2-y1;
By = y3-y4;

if(Ay<0) {						/* Y bound box test*/
  y1lo=(short)y2; y1hi=(short)y1;
  } else {
  y1hi=(short)y2; y1lo=(short)y1;
  }
if(By>0) {
  if(y1hi < (short)y4 || (short)y3 < y1lo) return DONT_INTERSECT;
  } else {
  if(y1hi < (short)y3 || (short)y4 < y1lo) return DONT_INTERSECT;
  }


Cx = x1-x3;
Cy = y1-y3;
d = By*Cx - Bx*Cy;					/* alpha numerator*/
f = Ay*Bx - Ax*By;					/* both denominator*/
if(f>0) {						/* alpha tests*/
  if(d<0 || d>f) return DONT_INTERSECT;
  } else {
  if(d>0 || d<f) return DONT_INTERSECT;
  }

e = Ax*Cy - Ay*Cx;					/* beta numerator*/
if(f>0) {						/* beta tests*/
  if(e<0 || e>f) return DONT_INTERSECT;
  } else {
  if(e>0 || e<f) return DONT_INTERSECT;
  }

/*compute intersection coordinates*/

if(f==0) return PARALLEL;
num = d*Ax;						/* numerator */
offset = SAME_SIGNS(num,f) ? f/2 : -f/2;		/* round direction*/
*x = x1 + (num+offset) / f;				/* intersection x */

num = d*Ay;
offset = SAME_SIGNS(num,f) ? f/2 : -f/2;
*y = y1 + (num+offset) / f;				/* intersection y */

return DO_INTERSECT;
}
