#include "sling_shot.h"

#include "global.h"
#include "fat.h"

SLING_SHOT::SLING_SHOT() : position(0), mx(0), my(0), power(0), fired(false), growing(false), diametre(MAX_DIAMETRE), fire_power(0)
{
}

void SLING_SHOT::logic()
{
    if (key[KEY_A])
    {
        if (position > SLING_MIN)
            position -= 5;
    }

    if (key[KEY_D])
    {
        if (position < SLING_MAX)
            position += 5;
    }


    if (fired)
    {
        if (power <= 0)
        {
            power = 0;
            diametre = 3;
            fired = false;
            growing = true;

            fixed angle, xo, yo;
            caculate_sling(30, 0, angle, xo, yo);
            angle = fixatan2(-xo, -yo);
            BALL t(position + fixtoi(xo), GAME_HEIGHT - 100 + fixtoi(yo), fire_power, angle);
            balls.add(t);
        }
        else
            power -= 10;
    }

    if (growing)
    {
        if (diametre < MAX_DIAMETRE)
        {
            diametre++;
        }
        else
        {
            growing = false;
        }
    }

    if ((!fired) && (!growing))
    {
        if (mouse_b & 1)
        {
            if (power < MAX_POWER)
                power += 3;
        }
        else
        {
            if (power > 0)
            {
                fire_power = power;
                fired = true;
            }
        }
    }
}


#include <math.h>
int tangent_points(double x1, double y1,
                               double x2, double y2, double r,
                               double *xi, double *yi,
                               double *xi_prime, double *yi_prime)
{
    int x_d, y_d, l;
    double c_x, c_y;

    x_d = (int)(x2 - x1);
    y_d = (int)(y2 - y1);
	l = (int)sqrt((double)((x_d * x_d) + (y_d * y_d))) / 2;
    c_x = x1 + (x_d / 2);
    c_y = y1 + (y_d / 2);

    return circle_circle_intersection(c_x, c_y, l, x2, y2, r, xi, yi, xi_prime, yi_prime);
}

void SLING_SHOT::caculate_sling(int p_base, int power, fixed &angle, fixed &xo, fixed &yo)
{
    int t = GAME_HEIGHT - mouse_y - 100;
    if (t > 200)
    {
        my = t;
    }

    int u = ORIGIN(mouse_x) - position;
    if (abs(u) > 100)
    {
        if (u < 0)
        {
            u = -100;
        }
        else
        {
            u = 100;
        }
    }
    mx = u;

    angle = fixatan2(mx, my);
    xo = -fixmul(itofix(30 + power), fixsin(angle));
    yo =  fixmul(itofix(30 + power), fixcos(angle));
}



#include <math.h>
void SLING_SHOT::render(BITMAP *buffer)
{
    fat_line(buffer, CENTRE(position) - 30, GAME_HEIGHT - 100, CENTRE(position) - 10, GAME_HEIGHT, 5, makecol(255,255,255));
    fat_line(buffer, CENTRE(position) + 30, GAME_HEIGHT - 100, CENTRE(position) + 10, GAME_HEIGHT, 5, makecol(255,255,255));

    fixed angle, xo, yo;
    caculate_sling(30, power, angle, xo, yo);

    circlefill(buffer, CENTRE(position) + fixtoi(xo), GAME_HEIGHT - 100 + fixtoi(yo), diametre, makecol(96,96,96));
    fat_circle(buffer, CENTRE(position) + fixtoi(xo), GAME_HEIGHT - 100 + fixtoi(yo), diametre, 1, makecol(255,0,0));

    int x1, y1, x2, y2;
    double a_x1, a_y1, a_x2, a_y2;
    double b_x1, b_y1, b_x2, b_y2;

    x2 = CENTRE(position) + fixtoi(xo);
    y2 = GAME_HEIGHT - 100 + fixtoi(yo);

    x1 = CENTRE(position) - 30;
    y1 = GAME_HEIGHT - 100;
    tangent_points(x1, y1, x2 - 1, y2, diametre, &a_x1, &a_y1, &a_x2, &a_y2);
    fat_line(buffer, x1, y1, a_x1, a_y1, 2, makecol(0, 0, 255));


    x1 = CENTRE(position) + 30;
    y1 = GAME_HEIGHT - 100;
    tangent_points(x1, y1, x2 + 1, y2, diametre, &b_x1, &b_y1, &b_x2, &b_y2);
    fat_line(buffer, x1, y1, b_x2, b_y2, 2, makecol(0, 0, 255));

    int l_dx = CENTRE(position) + fixtoi(xo) - a_x1;
    int l_dy = GAME_HEIGHT - 100 + fixtoi(yo) - a_y1;
    int r_dx = CENTRE(position) + fixtoi(xo) - b_x2;
    int r_dy = GAME_HEIGHT - 100 + fixtoi(yo) - b_y2;

    fixed l_a = fixatan2(itofix(l_dx), itofix(l_dy)) + itofix(64);
    fixed r_a = fixatan2(itofix(r_dx), itofix(r_dy)) + itofix(64);

    fat_arc(buffer, CENTRE(position) + fixtoi(xo), GAME_HEIGHT - 100 + fixtoi(yo), l_a, r_a, diametre, 2, makecol(0, 0, 255));


    textprintf_ex(buffer, font, 0, 10, makecol(255,255,255), makecol(0, 0, 0), "%d", position);
    textprintf_ex(buffer, font, 0, 20, makecol(255,255,255), makecol(0, 0, 0), "%d", mx);
    textprintf_ex(buffer, font, 0, 30, makecol(255,255,255), makecol(0, 0, 0), "%d", my);

    textprintf_ex(buffer, font, 500, 650, makecol(255,255,255), makecol(0, 0, 0), "Fired %d", fired);
}

